/**
 * 
 */
package com.wjholden.nmap;

import java.io.File;
import java.io.IOException;
import java.util.StringTokenizer;

import android.content.Context;
import android.content.pm.PackageManager.NameNotFoundException;
import android.os.Message;

/**
 * @author John
 *
 */
public class NmapUtilities implements Nmap_Constants {

	/**
	 * Removed from verifyInstallation for version 21.
	 * If temporary folder does not already exist, create it.
	 * @param c Context the application is executing in.
	 * @throws IOException
	 * @throws NameNotFoundException
	 */
	public static void verifyTemporaryFolder(Context c) throws IOException, NameNotFoundException
	{
		File tmpdir = new File(c.getPackageManager().getApplicationInfo("com.wjholden.nmap", 0).dataDir + "/tmp/");
		Nmap.ne.Log("Checking if " + tmpdir.getAbsolutePath() + " exists...");
		if (tmpdir.exists())
		{
			Nmap.ne.Log(tmpdir.getAbsolutePath() + " exists.");
		}
		else
		{
			Nmap.ne.Log(tmpdir.getAbsolutePath() + " does not exist. Creating it...");
			if (tmpdir.mkdirs())
			{
				Nmap.ne.Log(tmpdir.getAbsolutePath() + " created.");
			}
			else
			{
				Nmap.ne.Log("Something went wrong - " + tmpdir.getAbsolutePath() + " could not be created.");
				
				Message msg = Message.obtain();
				msg.obj = "Unable to create " + tmpdir.getAbsolutePath();
				msg.what = INSTALL_ERROR;
				Nmap.h.sendMessage(msg);
				return;
			}
		}
	}
	
	/**
	 * Mimics 'which' function to find 'su' command. Sets Nmap.sh and Nmap.canGetRoot.
	 * Originally, the program just looked for a hard-coded 'su' path, but CM6 has 'su'
	 * in a different location. Logic robusted to look everywhere in $PATH.
	 * @since 0.4.1
	 * @return true if su found in $PATH.
	 */
	public static boolean checkRootPermissions()
	{
		// should now be fixed @version 0.4.1 to allow /system/xbin/su on CM5+
		String path = System.getenv("PATH");
		StringTokenizer st = new StringTokenizer(path, ":");
		while (st.hasMoreTokens())
		{
			String token = st.nextToken();
			File suTest = new File(token + "/su");
			if (suTest.exists())
			{
				Nmap.canGetRoot = true;
				Nmap.setSh(suTest.getAbsolutePath());
				Nmap.ne.Log("su command found at " + Nmap.sh + " will run with root permissions.");
				return(true);
			}
		}
		
		// if you make it here, that means you couldn't get root, and there's no reason to worry about 'sh' being present.
		Nmap.canGetRoot = false;
		Nmap.setSh("sh");
		Nmap.ne.Log("su NOT found - will attempt to run without root permissions.");
		return(false);
	}
	
	/**
	 * Find the data directory to store applications in. Normally will be of form /data/da
	 * @param c
	 * @return
	 */
	public static String getDataDirectory(Context c)
	{
		try {
			return(c.getPackageManager().getApplicationInfo("com.wjholden.nmap", 0).dataDir);
		} catch (NameNotFoundException e) {
			Nmap.ne.Log(e.getMessage());
			return(null);
		}
	}
	
	/**
	 * Based on whether 'su' is available or not this method sets the location to write
	 * compiled binaries for later execution.
	 * @return Location to write binaries.
	 * @since 22 - UPDATE - now ALWAYS uses the {dataDir}/bin/ folder
	 * @param myDataDir - directory set by (above) getDataDirectory(Context)
	 */
	public static String findBinDir(String myDataDir)
	{
		/*if (Nmap.canGetRoot)
			Nmap.setBinDir("/data/local/bin/");
		else
			Nmap.setBinDir(myDataDir + "/bin/"); */
	
		String myBindir = myDataDir + "/bin/";
		
		Nmap.ne.Log("Set bindir to: " + myBindir);
		return(myBindir);
	}
}
