package com.wjholden.nmap;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;

import android.content.Context;
import android.content.pm.PackageManager.NameNotFoundException;
import android.os.AsyncTask;
import android.os.Message;

/**
 * Perform the actual scan.
 * 
 * 0.4.5:
 * - moved many local variables to private visibility
 * - utilized previously unused NmapError class.
 * 
 * @author William John Holden (wjholden@gmail.com)
 * @version 0.4.5
 */
// Object[0]=String, Object[1]=Context
// Turns out the Context isn't even needed.
public class scan extends AsyncTask <Object, Void, Void> implements Nmap_Constants
{
	private String command;
	private String executionParameters;
	private Context c;
	
	private DataOutputStream os;
	private BufferedReader in;
	private BufferedReader err;
	private Process p;

	protected void onPreExecute()
	{
		switch (Nmap.scanType) {
		case SCANTYPE_NMAP:
			command = "nmap ";
			Nmap.outputArgs = new String(" -oA /sdcard/nmap ");
			break;
		case SCANTYPE_NPING:
			command = "nping ";
			break;
		case SCANTYPE_NDIFF:
			command = "ndiff ";
			break;
		case SCANTYPE_NCAT:
			command = "ncat ";
			break;
		}
		
		if (Nmap.canGetRoot)
		{
			Nmap.outputArgs += " --privileged ";
		}
		else
		{
			Nmap.outputArgs += " --unprivileged ";
		}
		
		command += Nmap.outputArgs;

		Nmap.ne.Log("Selected scan type: " + command + " (" + Nmap.scanType + ")");
	}

	protected void onPostExecute(Void v)
	{
		if (Nmap.pd != null)
		{
			Nmap.pd.dismiss();
		}
		Nmap.hasRunOneScan = true;
	}

	@Override
	protected Void doInBackground(Object... params) {
		executionParameters = (String) params[0];
		c = (Context) params[1];

		if (Nmap.canGetRoot)
			Nmap.ne.Log("Getting root...");
		
		try {
			NmapUtilities.verifyTemporaryFolder(c);
		} catch (NameNotFoundException e1) {
			Nmap.ne.Log(e1.getMessage());
		} catch (IOException e1) {
			Nmap.ne.Log(e1.getMessage());
		}
		
		try {
			/*if (Nmap.canGetRoot)
				p = Runtime.getRuntime().exec("su");
			else
				p = Runtime.getRuntime().exec("sh");*/
			/** @since 21 - try using Nmap.sh */
			p = Runtime.getRuntime().exec(Nmap.sh);
		} catch (IOException e) {
			Nmap.ne.Log(e.getMessage());
			Message msg = Message.obtain();
			msg.obj = "Unable to start shell: " + e.toString();
			msg.what = RUN_ERROR;
			Nmap.h.sendMessage(msg);
			return(null);
		}
		if (Nmap.canGetRoot && p != null)
			Nmap.ne.Log("Got root!");
		else if (p == null)
		{
			Nmap.ne.Log("Process p is null - did getRuntime().exec(Nmap.sh) fail?");
			Message msg = Message.obtain();
			msg.obj = "Process for executing scan is null. Please close this program and reopen it.";
			msg.what = RUN_ERROR;
			Nmap.h.sendMessage(msg);
			return(null);
		}
		
		os = new DataOutputStream(p.getOutputStream());
		in = new BufferedReader(new InputStreamReader(p.getInputStream()));
		err = new BufferedReader(new InputStreamReader(p.getErrorStream()));

		if (os == null)
			Nmap.ne.Log("DataOutputStream 'os' is null!");
		if (in == null)
			Nmap.ne.Log("BufferedReader 'in' is null!");
		if (err == null)
			Nmap.ne.Log("BufferedReader 'err' is null!");

		try {
			os.writeBytes("cd " + Nmap.bindir + "\n");
			Nmap.ne.Log("cd " + Nmap.bindir);
			os.flush();
			/* wtf? -h is not a special case, and you're not doing anything that different.
			 * Also, current code branching includes useless if statement.
			 */

			if (executionParameters.equals("-h"))
				os.writeBytes("./" + command + " -h\n");
			else
				if (Nmap.canGetRoot)
				{
					os.writeBytes("./" + command + executionParameters + "\n");
					Nmap.ne.Log("./" + command + executionParameters);
				}
				else
				{
					os.writeBytes("./" + command + executionParameters + "\n");
					Nmap.ne.Log("./" + command + executionParameters);
				}

			/** @since 21 Rewrite for shorter passage that makes common fucking sense */
			/* BEGIN @since 21 */
			/*{
				if (executionParameters == null)
					executionParameters = "";
				String myCompleteCommand = "./" + command + executionParameters + "\n"; 
				os.writeBytes(myCompleteCommand);
				Nmap.ne.Log(myCompleteCommand);
			}*/
			/* END @since 21 */
			os.flush();
			os.writeBytes("exit\n");
			Nmap.ne.Log("exit");
			os.flush();
			os.close();

			// Catch the errors first: 
			String error, errLine;
			error = errLine = "";
			while ((errLine = err.readLine()) != null)
			{
				Nmap.ne.Log(errLine);
				error += errLine;
			}
			if (error.length() > 0)
			{
				Message msg1 = Message.obtain();
				msg1.obj = "Error detected at runtime: " + error;
				msg1.what = RUN_ERROR;
				Nmap.h.sendMessage(msg1);
			}
		} catch (IOException e) {
			Nmap.ne.Log(e.getMessage());
			Message msg = Message.obtain();
			msg.obj = "IOException: " + e.toString();
			msg.what = RUN_ERROR;
			Nmap.h.sendMessage(msg);
		}

		String line;
		try {
			while ((line = in.readLine()) != null)
			{
				Message msg = Message.obtain();
				msg.obj = line;
				msg.what = RUN_LINE;
				Nmap.h.sendMessage(msg);
			}
		} catch (IOException e) {
			Message msg = Message.obtain();
			msg.obj = "Unable to read command output: " + e.toString();
			msg.what = RUN_ERROR;
			Nmap.h.sendMessage(msg);
		}
		try {
			in.close();
			err.close();
		} catch (IOException e) {
			Message msg = Message.obtain();
			msg.obj = "Unable to close command output: " + e.toString();
			msg.what = RUN_ERROR;
			Nmap.h.sendMessage(msg);
		}

		p.destroy();

		if (!executionParameters.equals("-h"))
			Nmap.h.sendEmptyMessage(RUN_COMPLETE);
		return(null);
	}
}
