package com.wjholden.nmap;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;

import android.content.Context;
import android.content.res.Resources;
import android.os.AsyncTask;
import android.os.Message;

/**
 * Verifies the installation of Nmap binaries asynchronously.
 * Checks whether root permissions are available or not (Nmap.canGetRoot).
 * Creates the binary directory (Nmap.bindir).
 * 
 * TODO: Use md5 sums of files to ensure integrity. Maybe better as a user-driven process?
 * 
 * @author William John Holden (wjholden@gmail.com)
 * @version 0.4.6
 * @since 0.4.6 COMPLETE REWRITE
 */
public class verifyInstallation extends AsyncTask <Context, Void, Void> implements Nmap_Constants
{
	/**
	 * 
	 */
	private String installationResults;
	
	/**
	 * 
	 */
	private String filenames[] = { "ndiff", "nping", "nmap-services", "nmap-mac-prefixes" };
	
	/**
	 * 
	 */
	private int requiredResources[] = { R.raw.ndiff, R.raw.nping, R.raw.nmap_services, R.raw.nmap_mac_prefixes };
	
	/**
	 * 
	 */
	private Context context;
	
	private Resources resources;
	
	protected void onPostExecute(Void v)
	{
		Nmap.pd.dismiss();
		Nmap.installationVerified = true;
	}

	@Override
	protected Void doInBackground(Context... params)
	{
		byte[] buf = new byte[BUFFER_SIZE];
		if (params[0] != null)
			context = params[0];
		else
			return null;
		
		resources = context.getResources();
		Nmap.bindir = NmapUtilities.findBinDir(NmapUtilities.getDataDirectory(context.getApplicationContext()));

		// first write Nmap to file
		try
		{
			createNewFileAndDeleteExisting(Nmap.bindir + "/nmap");
			
			InputStream in_nmap_a = resources.openRawResource(R.raw.nmap_a);
			InputStream in_nmap_b = resources.openRawResource(R.raw.nmap_b);
			InputStream in_nmap_c = resources.openRawResource(R.raw.nmap_c);
			OutputStream out_nmap = new FileOutputStream(Nmap.bindir + "/nmap");
			while (in_nmap_a.read(buf) > 0)
			{
				out_nmap.write(buf);
			}
			in_nmap_a.close();
			while (in_nmap_b.read(buf) > 0)
			{
				out_nmap.write(buf);
			}
			in_nmap_b.close();
			while (in_nmap_c.read(buf) > 0)
			{
				out_nmap.write(buf);
			}
			in_nmap_c.close();
			out_nmap.close();
		}
		catch (IOException e)
		{
			Message msg = Message.obtain();
			msg.obj = "Error writing Nmap binary: " + e.toString();
			msg.what = INSTALL_ERROR;
			Nmap.h.sendMessage(msg);
			return(null);
		}
		
		// now write ncat
		try
		{
			createNewFileAndDeleteExisting(Nmap.bindir + "/ncat");
			
			InputStream in_ncat_a = resources.openRawResource(R.raw.ncat_a);
			InputStream in_ncat_b = resources.openRawResource(R.raw.ncat_b);
			OutputStream out = new FileOutputStream(Nmap.bindir + "/ncat");
			while (in_ncat_a.read(buf) > 0)
				out.write(buf);
			in_ncat_a.close();
			while (in_ncat_b.read(buf) > 0)
				out.write(buf);
			in_ncat_b.close();
			out.close();
		}
		catch (IOException e)
		{
			Message msg = Message.obtain();
			msg.obj = "Error writing Ncat binary: " + e.toString();
			msg.what = INSTALL_ERROR;
			Nmap.h.sendMessage(msg);
			return(null);
		}
		
		// write the remaining required resources, as defined in the requiredResources array
		for (int k = 0; k < requiredResources.length; k++)
		{
			try
			{
				createNewFileAndDeleteExisting(Nmap.bindir + "/" + filenames[k]);
				
				InputStream in1 = resources.openRawResource(requiredResources[k]);
				OutputStream out = new FileOutputStream(Nmap.bindir + "/" + filenames[k]);
				while (in1.read(buf) > 0)
				{
					out.write(buf);
				}
				in1.close();
				out.close();
			}
			catch (IOException e)
			{
				Message msg = Message.obtain();
				msg.obj = "Error writing " + filenames[k] + ": " + e.toString();
				msg.what = INSTALL_ERROR;
				Nmap.h.sendMessage(msg);
				return(null);
			}
		}
		
		// finally, use chmod command to make these babies executable
		try
		{
			//java.io.File file_nmap = new java.io.File(Nmap.bindir + "/nmap");
			//file_nmap.setExecutable(true); // not implemented? WTF?!!!
			
			Process p = Nmap.canGetRoot ? Runtime.getRuntime().exec("su") :  Runtime.getRuntime().exec("sh");
			DataOutputStream os = new DataOutputStream(p.getOutputStream());
			BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()), BUFFER_SIZE);
			BufferedReader err = new BufferedReader(new InputStreamReader(p.getErrorStream()), BUFFER_SIZE);
			os.writeBytes("cd " + Nmap.bindir + "\n");
			/*if (Nmap.canGetRoot)
			{
				os.writeBytes("chown root.root nmap\n");
				Nmap.ne.Log("chown root.root nmap");
				os.writeBytes("chown root.root nping\n");
				Nmap.ne.Log("chown root.root nping");
				os.writeBytes("chown root.root ncat\n");
				Nmap.ne.Log("chown root.root ncat");
			}
			os.writeBytes("chmod 555 nmap\n");
			Nmap.ne.Log("chmod 555 nmap");
			os.writeBytes("chmod 555 nping\n");
			Nmap.ne.Log("chmod 555 nping");
			os.writeBytes("chmod 555 nping\n");
			Nmap.ne.Log("chmod 555 nping");
			
			os.writeBytes("chmod 444 nmap-services\n");
			Nmap.ne.Log("chmod 444 nmap-services");
			os.writeBytes("chmod 444 nmap-mac-prefixes\n");
			Nmap.ne.Log("chmod 444 nmap-mac-prefixes");
			os.writeBytes("chmod 444 ndiff\n");
			Nmap.ne.Log("chmod 444 ndiff");*/
			
			if (Nmap.canGetRoot)
			{
				os.writeBytes("chown root.root *\n");
				Nmap.ne.Log("chown root.root *");
			}
			os.writeBytes("chmod 777 *\n");
			Nmap.ne.Log("chmod 777 *");
			
			os.writeBytes("exit\n");
			Nmap.ne.Log("exit");
			
			String s, e;
			while ((s = in.readLine()) != null)
			{
				installationResults += s + "\n";
			}
			while ((e = err.readLine()) != null)
			{
				installationResults += e + "\n";
			}
			
			os.close();
			in.close();
			err.close();
			p.waitFor();
			p.destroy();
			
			if (installationResults != null && !installationResults.equals(""))
			{
				Message msg = Message.obtain();
				msg.obj = installationResults;
				msg.what = INSTALL_ERROR;
				Nmap.h.sendMessage(msg);
			}
		}
		catch (IOException e)
		{
			Message msg = Message.obtain();
			msg.obj = "Error executing system command: " + e.toString() + " (this won't ever work in an emulator)";
			msg.what = INSTALL_ERROR;
			Nmap.h.sendMessage(msg);
			return(null);
		} catch (InterruptedException e) {
			Message msg = Message.obtain();
			msg.obj = e.toString();
			msg.what = INSTALL_ERROR;
			Nmap.h.sendMessage(msg);
			return(null);
		}
		
		// if you make to here that means there's absolutely no error so blast that message up and keep going.
		Nmap.h.sendEmptyMessage(INSTALL_GOOD);
		Nmap.ne.Log("Installation successful.");
		return(null);
	}
	
	private void createNewFileAndDeleteExisting(String path)
	{
		File f = new File (path);
		if (f.exists())
		{
			Nmap.ne.Log(path + " already exists. Deleting...");
			if (f.delete())
			{
				Nmap.ne.Log("\tdeleted.");
			}
			else
			{
				Nmap.ne.Log("\tunable to delete.");
			}
		}
	}
}
